(function( $, document ) {

	var iconic_wqt = {

		/**
		 * Set up cache with common elements and vars.
		 */
		cache: function() {
			iconic_wqt.tpl = {};
			iconic_wqt.els = {};
			iconic_wqt.vars = {};
			iconic_wqt.classes = {};

			// TPL
			iconic_wqt.tpl.tray = wp.template( 'iconic-wqt-tray' );

			// ELS
			iconic_wqt.els.tray = null;
			// @TODO - can we improve this selector?
			iconic_wqt.els.products_wrapper = $( '.products' );
			iconic_wqt.els.main_slider = null;
			iconic_wqt.els.thumb_slider = null;

			iconic_wqt.els.arrow_prev = '<div class="iconic-wqt-tray-slideshow__arrow iconic-wqt-tray-slideshow__arrow--prev"><svg id="Layer_1" viewBox="0 0 64 64" xmlns="http://www.w3.org/2000/svg"><g id="Icon-Arrow-Left"><path class="st0" d="M32 55.9C18.8 55.9 8.1 45.2 8.1 32S18.8 8.1 32 8.1 55.9 18.8 55.9 32 45.2 55.9 32 55.9zm0-45.2c-11.7 0-21.3 9.6-21.3 21.3S20.3 53.3 32 53.3 53.3 43.7 53.3 32 43.7 10.7 32 10.7z" id="Fill-25"/><path class="st0" id="Fill-26" d="M32.5 45.7L18.8 32l13.7-13.7 1.8 1.9L22.6 32l11.7 11.8-1.8 1.9"/><path class="st0" id="Fill-27" d="M20.7 30.6h24v2.8h-24z"/></g></svg></div>';
			iconic_wqt.els.arrow_next = '<div class="iconic-wqt-tray-slideshow__arrow iconic-wqt-tray-slideshow__arrow--next"><svg id="Layer_1" viewBox="0 0 64 64" xmlns="http://www.w3.org/2000/svg"><g id="Icon-Arrow-Right"><path class="st0" d="M32 55.9C18.8 55.9 8.1 45.2 8.1 32S18.8 8.1 32 8.1 55.9 18.8 55.9 32 45.2 55.9 32 55.9zm0-45.2c-11.7 0-21.3 9.6-21.3 21.3S20.3 53.3 32 53.3 53.3 43.7 53.3 32 43.7 10.7 32 10.7z" id="Fill-25"/><path class="st0" id="Fill-26" d="M31.5 45.7l-1.8-1.9L41.4 32 29.7 20.2l1.8-1.9L45.2 32 31.5 45.7"/><path class="st0" id="Fill-27" d="M43.3 33.4h-24v-2.8h24z"/></g></svg></div>';

			// VARS
			iconic_wqt.vars.tray_loading = false;
			iconic_wqt.vars.thumbs_to_show = 4;
			iconic_wqt.vars.window_size = iconic_wqt.get_window_size_sum();

			// CLASSES
			iconic_wqt.classes.active_row = 'iconic-wqt-tray-row-is-active';

		},

		/**
		 * Run on doc ready.
		 */
		on_ready: function() {
			iconic_wqt.cache();
			iconic_wqt.inject_quicktray();
			iconic_wqt.setup_triggers();
		},

		/**
		 * Run on window resize.
		 *
		 * We also check if the window size has actually changed,
		 * as the variations form triggers a resize for some reason.
		 */
		on_resize: function() {
			var window_size_sum = iconic_wqt.get_window_size_sum();

			if ( window_size_sum === iconic_wqt.vars.window_size ) {
				return;
			}

			iconic_wqt.vars.window_size = window_size_sum;
			iconic_wqt.close_tray();
		},

		/**
		 * Inject QuickTray before products wrapper.
		 */
		inject_quicktray: function() {
			if ( iconic_wqt.els.products_wrapper.length <= 0 ) {
				return;
			}

			iconic_wqt.els.products_wrapper.each( function( index, products_wrapper ) {
				var $products_wrapper = $( products_wrapper );

				if ( ! $products_wrapper.has( '.product' ) ) {
					return false;
				}

				$products_wrapper.before( '<div class="iconic-wqt-tray"></div>' );
				iconic_wqt.els.tray = $( '.iconic-wqt-tray' );

				return false;
			} );
		},

		/**
		 * Get products.
		 */
		get_products: function() {
			return iconic_wqt.els.products_wrapper.children( '.product' );
		},

		/**
		 * Setup tray triggers.
		 */
		setup_triggers: function() {
			$( document.body ).on( 'click', '[data-iconic-wqt-product-id]', function() {
				if ( iconic_wqt.vars.tray_loading ) {
					return;
				}

				iconic_wqt.vars.tray_loading = true;

				var $trigger = $( this ),
					product_id = parseInt( $trigger.data( 'iconic-wqt-product-id' ) ),
					button_text = $trigger.text();

				$trigger.text( iconic_wqt_vars.i18n.loading );

				iconic_wqt.get_product_data( product_id, function( response ) {
					if ( ! response.success ) {
						$trigger.text( button_text );
						return;
					}

					iconic_wqt.close_tray( $trigger, response, function() {
						iconic_wqt.position_tray( $trigger );
						iconic_wqt.populate_tray( response.data );
						iconic_wqt.open_tray( $trigger );
						$trigger.text( button_text );
						iconic_wqt.vars.tray_loading = false;
					} );
				} );
			} );

			$( document ).on( 'click', '.js-iconic-wqt-close', function( e ) {
				e.preventDefault();

				iconic_wqt.close_tray();
			} );
		},

		/**
		 * Get product data.
		 *
		 * @param product_id
		 * @param callback
		 */
		get_product_data: function( product_id, callback ) {
			var data = {
				'action': 'iconic_wqt_get_product_data',
				'product_id': product_id
			};

			jQuery.post( woocommerce_params.ajax_url, data, function( response ) {
				if ( typeof callback === "function" ) {
					callback( response );
				}
			} );
		},

		/**
		 * Populate tray.
		 *
		 * @param data
		 */
		populate_tray: function( data ) {
			iconic_wqt.els.tray.html( iconic_wqt.tpl.tray( data ) );

			var $variations_form = iconic_wqt.els.tray.find( '.variations_form' );

			if ( $variations_form.length > 0 ) {
				$variations_form.wc_variation_form();
			}
		},

		/**
		 * Position tray.
		 *
		 * @param $trigger
		 */
		position_tray: function( $trigger ) {
			iconic_wqt.els.$product = $trigger.closest( '.product' );
			iconic_wqt.vars.product_offset_top = iconic_wqt.els.$product.position().top;

			var product_height = iconic_wqt.get_tallest_height_row( false ),
				product_offset_bottom = iconic_wqt.vars.product_offset_top + product_height,
				container_width = iconic_wqt.els.tray.parent().width();

			iconic_wqt.els.tray.css( {
				'position': 'absolute',
				'top': product_offset_bottom,
				'width': container_width
			} );
		},

		/**
		 * Find tallest in row.
		 */
		get_tallest_height_row: function( include_margin ) {
			include_margin = typeof include_margin === 'undefined' ? true : include_margin;

			var tallest = 0,
				$products = iconic_wqt.get_products();

			$products.each( function() {
				if ( ! iconic_wqt.is_active_row( $( this ) ) ) {
					return;
				}

				var height = $( this ).outerHeight( include_margin );

				if ( height < tallest ) {
					return;
				}

				tallest = height;
			} );

			return tallest;

		},

		/**
		 * If tray is already open, check if is same row as trigger.
		 */
		is_this_row: function( $trigger ) {
			if ( $trigger.parents( '.product' ).hasClass( iconic_wqt.classes.active_row ) ) {
				return true;
			} else {
				return false;
			}
		},

		/**
		 * Open tray.
		 */
		open_tray: function( $trigger ) {
			iconic_wqt.els.tray.show();
			iconic_wqt.setup_slideshow( function() {
				$( 'html, body' ).animate( { scrollTop: iconic_wqt.get_scroll_to_position() }, 250 );
				iconic_wqt.setup_tray_placeholder();
				iconic_wqt.els.tray.addClass( 'iconic-wqt-tray--open' );
			} );
		},

		/**
		 * Get scroll to position.
		 */
		get_scroll_to_position: function() {
			var window_height = $( window ).height(),
				tray_height = iconic_wqt.els.tray.outerHeight(),
				tray_top = iconic_wqt.els.tray.offset().top;

			return tray_height >= window_height ? tray_top : tray_top - ( ( window_height - tray_height ) / 2 );
		},

		/**
		 * Close tray.
		 */
		close_tray: function( $trigger, response, callback ) {
			if ( ! iconic_wqt.els.tray.hasClass( 'iconic-wqt-tray--open' ) ) {
				if ( typeof callback === 'function' ) {
					callback( $trigger, response );
				}

				return;
			}

			var $products = iconic_wqt.get_products();

			iconic_wqt.els.tray.removeClass( 'iconic-wqt-tray--open' );
			$products.removeClass( iconic_wqt.classes.active_row );
			$products.removeAttr( 'style' );

			setTimeout( function() {
				if ( iconic_wqt.els.main_slider !== null ) {
					iconic_wqt.els.main_slider.slick( 'unslick' );
				}

				if ( iconic_wqt.els.thumb_slider !== null ) {
					iconic_wqt.els.thumb_slider.slick( 'unslick' );
				}

				iconic_wqt.els.tray.hide().html( '' );

				if ( typeof callback === 'function' ) {
					callback( $trigger, response );
				}
			}, 500 );
		},

		/**
		 * Setup tray placeholder.
		 */
		setup_tray_placeholder: function() {
			var products = [],
				tray_height = iconic_wqt.els.tray.outerHeight( true ),
				tallest_height = iconic_wqt.get_tallest_height_row(),
				$products = iconic_wqt.get_products();

			$products.each( function() {
				var $self = $( this );

				// Check to see if matches row
				if ( ! iconic_wqt.is_active_row( $self ) ) {
					return;
				}

				var self_height = $self.outerHeight( true ),
					product = {
						'el': $self,
						'margin_height': ( tray_height + tallest_height ) - self_height
					};

				// Add to array so heights are calculated before any
				// margins were added (prevents flexbox issue).
				products.push( product );
			} );

			if ( products.length <= 0 ) {
				return;
			}

			$.each( products, function( index, product ) {
				product.el.css( 'margin-bottom', product.margin_height ).addClass( iconic_wqt.classes.active_row );
			} );
		},

		/**
		 * Check if passed element is in the active row.
		 *
		 * @param $element
		 *
		 * @return bool
		 */
		is_active_row: function( $element ) {
			return Math.round( $element.position().top ) === Math.round( iconic_wqt.vars.product_offset_top );
		},

		/**
		 * Setup slideshows.
		 */
		setup_slideshow: function( callback ) {

			iconic_wqt.els.main_slider = iconic_wqt.els.tray.find( '.iconic-wqt-tray-slideshow--main' );
			iconic_wqt.els.thumb_slider = iconic_wqt.els.tray.find( '.iconic-wqt-tray-slideshow--thumbnails' );

			iconic_wqt.els.thumb_slider.on( 'init', function() {
				if ( iconic_wqt.els.thumb_slider.find( '.iconic-wqt-tray-slideshow__slide' ).length <= 1 ) {
					iconic_wqt.els.thumb_slider.remove();
				}

				iconic_wqt.els.tray.find( '.iconic-wqt-tray__images' ).css( { opacity: 1 } );

				if ( typeof callback === 'function' ) {
					callback();
				}
			} );

			iconic_wqt.images_loaded( iconic_wqt.els.tray, function() {
				iconic_wqt.els.main_slider.slick( {
					speed: 250,
					prevArrow: iconic_wqt.els.arrow_prev,
					nextArrow: iconic_wqt.els.arrow_next,
					infinite: false,
					arrows: true,
					adaptiveHeight: true,
					responsive: [
						{
							breakpoint: 920,
							settings: {
								arrows: false
							}
						}
					]
				} );

				var slides_to_scroll = iconic_wqt.els.thumb_slider.children().length <= iconic_wqt.vars.thumbs_to_show ? iconic_wqt.vars.thumbs_to_show : 1;

				iconic_wqt.els.thumb_slider.slick( {
					speed: 250,
					infinite: false,
					arrows: false,
					slidesToShow: iconic_wqt.vars.thumbs_to_show,
					slidesToScroll: slides_to_scroll
				} );
			} );

			iconic_wqt.els.main_slider.on( 'beforeChange', function( event, slick, current_slide, next_slide ) {
				iconic_wqt.set_active_thumbnail( next_slide );
			} );

			iconic_wqt.els.thumb_slider.on( 'swipe', function( event, slick, direction ) {
				var $current_slide = iconic_wqt.els.thumb_slider.find( '.slick-current' ),
					index = $current_slide.data( 'slick-index' );

				iconic_wqt.els.main_slider.slick( 'slickGoTo', index );
			} );

			iconic_wqt.els.thumb_slider.on( 'click', '.iconic-wqt-tray-slideshow__slide', function() {
				var index = $( this ).data( 'slick-index' );

				iconic_wqt.set_active_thumbnail( index );

				iconic_wqt.els.main_slider.slick( 'slickGoTo', index );
			} );
		},

		/**
		 * Set active thumbnail.
		 *
		 * @param index
		 */
		set_active_thumbnail: function( index ) {
			var $thumbnail = iconic_wqt.els.thumb_slider.find( '[data-slick-index="' + index + '"]' ),
				thumbnail_slide_count = iconic_wqt.els.thumb_slider.find( '.iconic-wqt-tray-slideshow__slide' ).length,
				last_index = thumbnail_slide_count - iconic_wqt.vars.thumbs_to_show,
				go_to_index = index > last_index ? last_index : index;

			iconic_wqt.els.thumb_slider.slick( 'slickGoTo', go_to_index );
			iconic_wqt.els.thumb_slider.find( '.slick-current' ).removeClass( 'slick-current' );
			$thumbnail.addClass( 'slick-current' );
		},

		/**
		 * Get window size sum.
		 *
		 * @return int
		 */
		get_window_size_sum: function() {
			var $window = $( window );

			return $window.width() + $window.height();
		},

		/**
		 * Images loaded with srcset.
		 */
		images_loaded: function( selector, on_complete, on_progress ) {
			var $images = $( selector ).find( 'img' );
			var success = 0;
			var error = 0;
			var iteration = 0;
			var total = $images.length;
			var check = function( el, status ) {
				iteration ++;
				var data = {
					img: el,
					iteration: iteration,
					success: success,
					error: error,
					total: total,
					status: status
				};
				if ( $.isFunction( on_progress ) ) {
					on_progress( data );
				}
				if ( success + error === total ) {
					if ( $.isFunction( on_complete ) ) {
						on_complete( data );
					}
				}
			};
			$images.each( function() {
				this.onload = function() {
					success ++;
					check( this, 'success' );
				};
				this.onerror = function() {
					error ++;
					check( this, 'error' );
				};
			} );
		}
	};

	$( document ).ready( iconic_wqt.on_ready );
	$( window ).on( 'resize', iconic_wqt.on_resize );

}( jQuery, document ));