<?php
/**
 * Plugin Name: WooCommerce QuickTray
 * Plugin URI: https://iconicwp.com
 * Description: Load products in a tray directly in your WooCommerce catalog.
 * Version: 1.0.1
 * Author: Iconic <info@iconicwp.com>
 * Author URI: https://iconicwp.com
 * Text Domain: iconic-wqt
 * WC requires at least: 3.0.0
 * WC tested up to: 3.4.5
 */

if ( ! defined( 'WPINC' ) ) {
	wp_die();
}

class Iconic_Woo_QuickTray {

	/**
	 * Class prefix
	 *
	 * @since  1.0.0
	 * @access protected
	 * @var string $class_prefix
	 */
	protected $class_prefix = "Iconic_WQT_";

	/**
	 * Construct
	 */
	public function __construct() {
		$this->textdomain();
		$this->define_constants();
		$this->load_classes();
	}

	/**
	 * Load textdomain
	 */
	public function textdomain() {
		load_plugin_textdomain( 'iconic-wqt', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}

	/**
	 * Set constants
	 */
	public function define_constants() {
		$this->define( 'ICONIC_WQT_VERSION', '1.0.1' );
		$this->define( 'ICONIC_WQT_PATH', plugin_dir_path( __FILE__ ) );
		$this->define( 'ICONIC_WQT_URL', plugin_dir_url( __FILE__ ) );
		$this->define( 'ICONIC_WQT_INC_PATH', ICONIC_WQT_PATH . 'inc/' );
		$this->define( 'ICONIC_WQT_VENDOR_PATH', ICONIC_WQT_INC_PATH . 'vendor/' );
		$this->define( 'ICONIC_WQT_BASENAME', plugin_basename( __FILE__ ) );
	}

	/**
	 * Define constant if not already set.
	 *
	 * @param string      $name
	 * @param string|bool $value
	 */
	private function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
	 * Load classes
	 */
	private function load_classes() {
		require_once( ICONIC_WQT_INC_PATH . 'class-core-autoloader.php' );

		Iconic_WQT_Core_Autoloader::run( array(
			'prefix'   => 'Iconic_WQT_',
			'inc_path' => ICONIC_WQT_INC_PATH,
		) );

		if ( ! Iconic_WQT_Core_Helpers::is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			return;
		}

		Iconic_WQT_Core_Settings::run( array(
			'vendor_path'   => ICONIC_WQT_VENDOR_PATH,
			'title'         => 'WooCommerce QuickTray',
			'version'       => ICONIC_WQT_VERSION,
			'menu_title'    => 'QuickTray',
			'settings_path' => ICONIC_WQT_INC_PATH . 'admin/settings.php',
			'option_group'  => 'iconic_wqt',
			'docs'          => array(
				'collection'      => '/collection/204-woocommerce-quicktray',
				'troubleshooting' => '/category/209-troubleshooting',
				'getting-started' => '/category/207-getting-started',
			),
			'cross_sells'   => array(
				'iconic-woo-show-single-variations',
				'iconic-woo-attribute-swatches',
			),
		) );

		Iconic_WQT_Settings::run();
		Iconic_WQT_QuickTray::run();
		Iconic_WQT_Assets::run();
		Iconic_WQT_Ajax::run();
	}
}

$iconic_wqt = new Iconic_Woo_QuickTray();
