<?php
if ( ! defined( 'WPINC' ) ) {
	wp_die();
}

/**
 * Class Iconic_WQT_Ajax
 */
class Iconic_WQT_Ajax {
	/**
	 * Hook in ajax handlers.
	 */
	public static function run() {
		self::add_ajax_events();
	}

	/**
	 * Hook in methods - uses WordPress ajax handlers (admin-ajax).
	 */
	public static function add_ajax_events() {
		$ajax_events = array(
			'get_product_data' => true,
		);

		foreach ( $ajax_events as $ajax_event => $nopriv ) {
			add_action( 'wp_ajax_iconic_wqt_' . $ajax_event, array( __CLASS__, $ajax_event ) );
			if ( $nopriv ) {
				add_action( 'wp_ajax_nopriv_iconic_wqt_' . $ajax_event, array( __CLASS__, $ajax_event ) );
			}
		}
	}

	/**
	 * Get product data.
	 */
	public static function get_product_data() {
		global $product;

		$product_id = filter_input( INPUT_POST, 'product_id', FILTER_SANITIZE_NUMBER_INT );
		$product    = wc_get_product( $product_id );

		$json = array(
			'success' => false,
			'data'    => array(),
		);

		if ( ! $product ) {
			wp_send_json( $json );
		}

		$json['success'] = true;

		$template_args = array(
			'product'   => $product,
		);

		$json['data']['title']       = $product->get_title();
		$json['data']['price']       = $product->get_price_html();
		$json['data']['images']      = Iconic_WQT_Product::get_slides_html( $product, 'single' );
		$json['data']['thumbnails']  = Iconic_WQT_Product::get_slides_html( $product, 'thumbnail' );
		$json['data']['description'] = $product->get_short_description();
		$json['data']['meta']        = Iconic_WQT_Helpers::get_woocommerce_template_html( 'single-product/meta.php', $template_args );
		$json['data']['add_to_cart'] = Iconic_WQT_Helpers::get_woocommerce_template_html( 'add-to-cart', $template_args );
		$json['data']['rating']      = wc_get_rating_html( $product->get_average_rating() );

		wp_send_json( apply_filters( 'iconic_wqt_product_data', $json, $product ) );
	}
}