<?php
if ( ! defined( 'WPINC' ) ) {
	wp_die();
}

/**
 * Class Iconic_WQT_Assets
 */
class Iconic_WQT_Assets {
	/**
	 * Run.
	 */
	public static function run() {
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'frontend_scripts' ), 100 );
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'frontend_styles' ), 105 );
	}

	/**
	 * Load all frontend scripts.
	 */
	public static function frontend_scripts() {
		self::register_scripts();

		$enqueue = apply_filters( 'iconic_wqt_scripts', array(
			'jquery',
			'wp-util',
			'wc-add-to-cart-variation',
			'iconic-wqt-slick',
			'iconic-wqt-imagesloaded',
			'iconic-wqt',
		) );

		if ( empty( $enqueue ) ) {
			return;
		}

		foreach ( $enqueue as $handle ) {
			wp_enqueue_script( $handle );
		}
	}

	/**
	 * Register all scripts.
	 */
	private static function register_scripts() {
		$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		$register_scripts = array(
			'iconic-wqt'              => array(
				'src'     => ICONIC_WQT_URL . 'assets/frontend/js/main' . $suffix . '.js',
				'deps'    => array( 'jquery', 'wp-util' ),
				'version' => ICONIC_WQT_VERSION,
			),
			'iconic-wqt-slick'        => array(
				'src'     => ICONIC_WQT_URL . 'assets/vendor/js/slick' . $suffix . '.js',
				'deps'    => array( 'jquery' ),
				'version' => ICONIC_WQT_VERSION,
			),
		);

		foreach ( $register_scripts as $name => $props ) {
			wp_register_script( $name, $props['src'], $props['deps'], $props['version'], true );
		}

		wp_localize_script( 'iconic-wqt', 'iconic_wqt_vars', array(
			'i18n' => array(
				'loading' => __( 'Loading...', 'iconic-wqt' ),
			),
		) );
	}

	/**
	 * Load all frontend styles.
	 */
	public static function frontend_styles() {
		self::register_styles();

		$enqueue = apply_filters( 'iconic_wqt_styles', array(
			'iconic-wqt-default',
			'iconic-wqt-theme',
			'iconic-wqt-slick',
		) );

		if ( empty( $enqueue ) ) {
			return;
		}

		foreach ( $enqueue as $handle ) {
			wp_enqueue_style( $handle );
		}
	}

	/**
	 * Register all styles.
	 */
	private static function register_styles() {
		$register_styles = array(
			'iconic-wqt-default' => array(
				'src'     => ICONIC_WQT_URL . 'assets/frontend/css/default.css',
				'deps'    => array(),
				'version' => ICONIC_WQT_VERSION,
			),
			'iconic-wqt-theme'   => array(
				'src'     => ICONIC_WQT_URL . 'assets/frontend/css/theme.css',
				'deps'    => array(),
				'version' => ICONIC_WQT_VERSION,
			),
			'iconic-wqt-slick'   => array(
				'src'     => ICONIC_WQT_URL . 'assets/vendor/css/slick.css',
				'deps'    => array(),
				'version' => ICONIC_WQT_VERSION,
			),
		);

		foreach ( $register_styles as $name => $props ) {
			wp_register_style( $name, $props['src'], $props['deps'], $props['version'] );
		}
	}
}