<?php
if ( ! defined( 'WPINC' ) ) {
	wp_die();
}

/**
 * Class Iconic_WQT_QuickTray
 */
class Iconic_WQT_QuickTray {

	/**
	 * Run.
	 */
	public static function run() {
		add_action( 'wp_footer', array( __CLASS__, 'output_tray_template_script' ), 100 );
		add_action( 'woocommerce_after_shop_loop_item', array( __CLASS__, 'output_button' ), 100 );
		add_filter( 'woocommerce_add_to_cart_form_action', array( __CLASS__, 'add_to_cart_form_action' ), 10 );

		self::setup_tray_components();
	}

	/**
	 * Output the tray template.
	 */
	public static function output_tray_template_script() {
		?>
		<script type="text/html" id="tmpl-iconic-wqt-tray">
			<?php do_action( 'iconic_wqt_tray' ); ?>
		</script>
		<?php
	}

	/**
	 * Output the button template.
	 */
	public static function output_button() {
		Iconic_WQT_Helpers::include_template( 'button' );
	}

	/**
	 * Setup tray components using hooks.
	 */
	public static function setup_tray_components() {
		$components = apply_filters( 'iconic_wqt_template_components', array(
			'images_open'   => 10,
			'images'        => 20,
			'thumbnails'    => 30,
			'images_close'  => 40,
			'summary_open'  => 50,
			'close'         => 60,
			'title'         => 70,
			'price'         => 80,
			'description'   => 90,
			'add_to_cart'   => 100,
			'meta'          => 110,
			'rating'        => 120,
			'summary_close' => 130,
		) );

		foreach ( $components as $component => $priority ) {
			$method_name = sprintf( 'output_tray_%s', $component );

			if ( ! method_exists( __CLASS__, $method_name ) ) {
				continue;
			}

			add_action( 'iconic_wqt_tray', array( __CLASS__, $method_name ), $priority );
		}
	}

	/**
	 * Output summary open.
	 */
	public static function output_tray_summary_open() {
		echo '<div class="iconic-wqt-tray__summary">';
	}

	/**
	 * Output summary close.
	 */
	public static function output_tray_summary_close() {
		echo '</div>';
	}

	/**
	 * Output images open.
	 */
	public static function output_tray_images_open() {
		echo '<div class="iconic-wqt-tray__images">';
	}

	/**
	 * Output images close.
	 */
	public static function output_tray_images_close() {
		echo '</div>';
	}

	/**
	 * Output close element.
	 */
	public static function output_tray_close() {
		echo '<div class="iconic-wqt-tray__close"><a href="#" class="js-iconic-wqt-close"><img src="' . ICONIC_WQT_URL . 'assets/frontend/images/svg/close.svg" alt="Close Tray" /></a></div>';
	}

	/**
	 * Output the tray title.
	 */
	public static function output_tray_title() {
		Iconic_WQT_Helpers::include_template( 'tray/title' );
	}

	/**
	 * Output the tray price.
	 */
	public static function output_tray_price() {
		Iconic_WQT_Helpers::include_template( 'tray/price' );
	}

	/**
	 * Output the tray images.
	 */
	public static function output_tray_images() {
		Iconic_WQT_Helpers::include_template( 'tray/images' );
	}

	/**
	 * Output the tray thumbnails.
	 */
	public static function output_tray_thumbnails() {
		Iconic_WQT_Helpers::include_template( 'tray/thumbnails' );
	}

	/**
	 * Output the tray description.
	 */
	public static function output_tray_description() {
		Iconic_WQT_Helpers::include_template( 'tray/description' );
	}

	/**
	 * Output the tray meta.
	 */
	public static function output_tray_meta() {
		Iconic_WQT_Helpers::include_template( 'tray/meta' );
	}

	/**
	 * Output the tray add to cart.
	 */
	public static function output_tray_add_to_cart() {
		Iconic_WQT_Helpers::include_template( 'tray/add-to-cart' );
	}

	/**
	 * Output the tray rating.
	 */
	public static function output_tray_rating() {
		Iconic_WQT_Helpers::include_template( 'tray/rating' );
	}

	/**
	 * Change add to cart redirect in archive.
	 *
	 * @param string $form_action
	 *
	 * @return string
	 */
	public static function add_to_cart_form_action( $form_action ) {
		$action = filter_input( INPUT_POST, 'action' );

		if ( $action !== 'iconic_wqt_get_product_data' ) {
			return $form_action;
		}

		return '';
	}
}